import { getValueByPath, ensureValueByPath } from '../../src/util'

type Data = Record<string, any>

const KEY_MAP = {
    'Form': {
        "select": {
            "meta": {
                "data": "..items"
            },
            // "placeholder": "meta.placeholder"
        },
        "meta": {
            // "show": "..show",
            "label": "..label",
        },
    },

    // "disabled": ".meta.disabled"
}
const SCHEMA_TYPES = ['Form', 'Button', 'Modal', 'Table', 'Tabs', 'Iframe', 'Panel', 'select', 'input', 'operations', 'textArea', 'codeEditor', 'date', 'switch', 'checkbox', 'radio']
const SCHEMA_TYPES_REG = new RegExp(`\^((${SCHEMA_TYPES.join('|')})\\.)+`)
const FIRST_SCHEMA_TYPE_REG = new RegExp(`\^((${SCHEMA_TYPES.join('|')})\\.)`)


const resolveRelativePath = (sourcePath: string, relativePath: string) => {
    const sourcePathArr = sourcePath.split('.')
    const relativePathArr = relativePath.split('.')

    while (relativePathArr.length) {
        const current = relativePathArr.shift()

        if(!current){
            sourcePathArr.splice(-1)
        }else{
            sourcePathArr.push(current)
        }
    }

    console.log(`resolveRelativePath sourcePath: '${sourcePath}' and relativePath: '${relativePath}' to ${sourcePathArr.join('.')}`)

    return sourcePathArr.join('.')
}

const evalMappedKeysByPath = (path: string): string | undefined => {
    let mappedPath = getValueByPath(path, KEY_MAP) as string

    while (!mappedPath && path && FIRST_SCHEMA_TYPE_REG.test(path)) {
        path = path.replace(FIRST_SCHEMA_TYPE_REG, '')
        mappedPath = getValueByPath(path, KEY_MAP) as string
    }
    return mappedPath
}

const getMappedPath = (path: string): string => {
    let mappedPath = evalMappedKeysByPath(path) as string

    if(!mappedPath) return path.replace(SCHEMA_TYPES_REG, '')

    const relativePathReg = /^\.+/
    const isRelativePath = relativePathReg.test(mappedPath)

    if(isRelativePath){
        mappedPath = resolveRelativePath(path, mappedPath)
    }

    return mappedPath.replace(SCHEMA_TYPES_REG, '')
}

function getFullPath(path: string, parentPath?: string): string{
    return parentPath ? `${parentPath}.${path}` : path
}

function setNewSchema(newSchema: Data, key: string, value: any, parentTypePath: string){
    const sourcePath = getFullPath(key, parentTypePath)
    const mappedPath = getMappedPath(sourcePath)

    ensureValueByPath(newSchema, mappedPath, value)

    return newSchema
}

function mapSchemaMeta(newSchema: any, meta: Data, parentTypePath: string): any{
    Object.entries(meta).forEach(([key, value]) => {
        setNewSchema(newSchema, key, value, getFullPath('meta', parentTypePath))
    })

    return newSchema
}

function mapSchema(oldSchema: any, parentTypePath: string = ''): any{
    const {type} = oldSchema // table的items没有type
    const pathArr = []

    if(parentTypePath) pathArr.push(parentTypePath)
    if(type) pathArr.push(type)

    const typePath = pathArr.join('.')

    const newSchema = Object.entries(oldSchema).reduce( (newSchema, [key, value]) => {
        if(key === 'meta') return mapSchemaMeta(newSchema, value as Data, typePath)
        if(key === 'items') return setNewSchema(newSchema, key, mapSchemas(value as any[], typePath), parentTypePath)

        return setNewSchema(newSchema, key, value, typePath)
    }, {})

    return newSchema
}

function mapSchemas(oldSchemas: any[], parentTypePath: string = ''): any[]{
    return oldSchemas.map( oldSchema => mapSchema(oldSchema, parentTypePath) )
}

export default function map(oldSchemas: any[]): any{
    return mapSchemas(oldSchemas)
}