import { ActionPlugin, Action } from '../manager/action'
import { PluginTypes, ActionTypes } from '../constant'
import { Data } from '../schema'
import { Util } from '../plugins'
import { DialogMethods } from '../parsers/dialog'
import { ComponentPublicInstance } from 'vue'
import { RendererProps } from '../components/renderer'
import logger from '../util/logger'

export type OpenDialogMeta = {
    modal?: string,
    params?: Data
}

export default {
    name: ActionTypes.OpenDialog,
    type: PluginTypes.Action,
    specificMeta: 'modal',
    dependentMetas: ['modal', 'params'] as Array<keyof OpenDialogMeta>,
    handler({
        modal,
        params,
    }: Action<OpenDialogMeta>, {
        pluginDriver
    }: Util){
        const dialog = modal && pluginDriver.getComponent(modal) as ComponentPublicInstance<RendererProps, {}, {}, {}, DialogMethods>

        if(!dialog) {
            logger.error(`Action ${ActionTypes.OpenDialog} can't find component instance of key: ${modal}`)
            return
        }

        params && dialog.setSource(params)
        dialog.open()

    }
} as ActionPlugin