import { ActionPlugin, Action } from '../manager/action'
import { PluginTypes, ActionTypes } from '../constant'
import { Data } from '../schema'
import { Util } from '../plugins'
import { TableMethods } from '../parsers/table'
import { ComponentPublicInstance } from 'vue'
import { RendererProps } from '../components/renderer'
import logger from '../util/logger'
import { isObject } from '../util'

// 表单提交后，将数据作为table的筛选项
export type FilterMeta = {
    filter?: string,
}

export default {
    name: ActionTypes.Filter,
    type: PluginTypes.Action,
    specificMeta: 'filter',
    dependentMetas: ['filter'] as Array<keyof FilterMeta>,
    handler({
        filter
    }: Action<FilterMeta>, {
        pluginDriver
    }: Util, model: Data){
        const component = filter && pluginDriver.getComponent(filter)

        if(!component) {
            logger.error(`Action ${ActionTypes.Filter} can't find component instance of key: ${filter}`)
            return
        }

        if(isObject(model)){
            (component as ComponentPublicInstance<RendererProps, {}, {}, {}, TableMethods>).setExtendParams(model)
        }

        (component as ComponentPublicInstance<RendererProps, {}, {}, {}, TableMethods>).refresh()
    }
} as ActionPlugin