import { ActionPlugin, Action } from '../manager/action'
import { PluginTypes, ActionTypes } from '../constant'
import { Data } from '../schema'
import { Util } from '../plugins'

export type Alias = Record<string, string>

export type RequestMeta = {
    url?: string,
    method?: 'post' | 'POST' | 'get' | 'GET',
    params?: Data,
    data?: Data,
    alias?: Alias,
    requestPlugin?: string,
}

export const enum RequestHooks {
    OnSuccess = 'onSuccess'
}

export default {
    name: ActionTypes.Request,
    type: PluginTypes.Action,
    specificMeta: 'url',
    dependentMetas: ['url', 'method', 'params', 'data', 'alias', 'requestPlugin'] as Array<keyof RequestMeta>,
    handler(requestMeta: Action<RequestMeta>, {
        pluginDriver,
        http
    }: Util){
        const {requestPlugin, ...meta} = requestMeta

        return http(meta).then( (res: Data) => {
            if(requestPlugin) res = pluginDriver.runHook(requestPlugin, RequestHooks.OnSuccess, this, res)

            return res
        })
    }
} as ActionPlugin