import { ActionPlugin, Action } from '../manager/action'
import { PluginTypes, ActionTypes } from '../constant'
import { Util } from '../plugins'
import { FormUIMethods, FormMethods } from '../parsers/form'
import { ComponentPublicInstance } from 'vue'
import logger from '../util/logger'

export type SubmitMeta = {
    form?: string,
}

export default {
    name: ActionTypes.Submit,
    type: PluginTypes.Action,
    specificMeta: 'form',
    dependentMetas: ['form'] as Array<keyof SubmitMeta>,
    handler({
        form
    }: Action<SubmitMeta>, {
        pluginDriver
    }: Util): Promise<any> | void{
        if(!form) return logger.error(`Action ${ActionTypes.Submit} required form key, but get: ${form}`)

        const component = pluginDriver.getComponent(form)

        if(!component) return logger.error(`Action ${ActionTypes.Submit} can't find component instance of key: ${form}`)

        const ui = pluginDriver.getUIInstance(form)

        if(!ui) return logger.error(`Action ${ActionTypes.Submit} can't find UI instance of key: ${form}`)

        return (ui as ComponentPublicInstance<{}, {}, {}, {}, FormUIMethods>).validate().then( () => (component as ComponentPublicInstance<{}, {}, {}, {}, FormMethods>).submit()).catch( reason => {
            logger.warn(`Action ${ActionTypes.Submit} error: `, reason )
        })
    }
} as ActionPlugin