import { RendererProps, ParserUtil } from '../renderer'
import { pluginDriver } from '../../plugins'
import { SetupContext, getCurrentInstance } from 'vue'
import { ComponentNode } from '../../schema'

// 调用解析器parse，返回初始化的ComponentNode，并提供重新渲染组件的方法

export default function useParser(schema: RendererProps, context: SetupContext, parserUtil: ParserUtil){
    const {
        isInit,
        component,
        data,
        props,
        events,
        children,
        methods = {},
    }: ComponentNode = pluginDriver.parseSchema(schema, context, parserUtil)

    // 将组件实例增加到ComponentManager里
    pluginDriver.addComponent(schema.uniqueKey, getCurrentInstance()?.proxy)

    return {
        isInit,
        component,
        props,
        data,
        events,
        children,
        methods,
    }
}