import { App, createApp } from 'vue'
import { UserPlugins, PluginDriver } from './plugins'
import { UIConfig } from './manager/ui'
import router from './router'
import { SchemaRenderer } from './components/renderer'
import { ParserTypes } from './constant'
import { http } from './util/http'
import { normalizeSchema } from './util'
import { BaseSchema, SchemaNode } from './schema'

// 建站平台应用相关配置信息
export type AppConfig = {
    appKey: string,
    title: string,
    kcpApi: string,
    env: string,
    pageApiPath: string,
    menuApiPath: string,
    loginUrl: string,
}

export type Config = {
    ui: UIConfig
    appConfig: AppConfig
    plugins?: UserPlugins
}

export default class Engine {
    app?: App
    // 引擎配置信息
    options: Config
    pluginDriver: PluginDriver | null
    constructor(options: Config) {
        this.options = options
        this.pluginDriver = null

        this.init()
    }

    init(){
        this.initPluginDriver()
        this.createApp()
    }

    initPluginDriver(){
        const {options} = this
        const pluginDriver =  this.pluginDriver = PluginDriver.getInstance()

        pluginDriver.setOptions(options)
    }

    getApp(): App{
        return this.app as App
    }

    createApp(){
        const {
            appConfig
        } = this.options
        const {
            appKey: key,
            title,
        } = appConfig
        const frameProps: BaseSchema = {
            key,
            type: ParserTypes.Frame,
            meta: {
                title
            },
        }
        const app = createApp(SchemaRenderer, normalizeSchema(frameProps as SchemaNode) as any)
        const {globalProperties} = app.config
        const engineGlobalProperties = {
            http,
            appConfig,
        }

        Object.entries(engineGlobalProperties).forEach(([key, methods]) => {
            globalProperties[`$${key}`] = methods
        })

        app.use(router)

        this.app = app
    }
}