import { toRawType, isObject, isArray, isString } from '../util'
import { Data } from '../schema'

export type EvaluatorOptions = {
    source: Data,
    alias?: Data<string>
}

export abstract class BaseEvaluator {
    options: EvaluatorOptions | null // Evaluator配置

    constructor(){
        this.options = null
    }

    abstract setOptions(options: any): void

    eval(source: unknown): any{
        if(isArray(source) ){
            return source.map((item) => this.eval(item))
        }else if(isObject(source)){
            return Object.entries(source).reduce((result, [key, value]) => {
                const valueType = toRawType(value)
                const hasValue = valueType !== 'Undefined' && valueType !== 'Null'
                const isObjectOrArray = valueType === 'Array' || valueType === 'Object'

                if(!hasValue) return result

                let convertedValue

                if(isObjectOrArray){
                    convertedValue = this.eval(value)
                }else if(valueType === 'String'){
                    convertedValue = this.evalString(value)
                }else{
                    convertedValue = value
                }

                return Object.assign(result, {
                    [key]: convertedValue
                })
            }, {})
        }else if( isString(source) ){
            return this.evalString(source)
        }else{
            return source
        }
    }

    /**
     * 解析带有表达式的字符串
     *
     * @param   {String}             string        输入的字符串
     * @param   {EvaluatorOptions}   options       Evaluator配置
     *
     * @return  {String}                           转换后的字符串
     */
     evalString(string: string){
        const hasVariable = string.indexOf('{') > -1

        if (!hasVariable) return this.evalItem(string)

        return string.replace(/\{(.+)\}/g, (matched, expression) => this.evalItem(expression))
    }

    abstract evalItem(source: string): any
}