import { Data } from '../schema'
import { BaseEvaluator, EvaluatorOptions } from './base'
import { isArray, isPlainObject, getValueByPath } from '../util/index'

export default class PathEvaluator extends BaseEvaluator {
    alias: Data<string>
    variableFlag: RegExp
    sourceData: Data | null // 取值目标数据源
    constructor(){
        super()
        this.sourceData = null
        this.variableFlag = /^\$/
        this.alias = {}
    }

    setAlias(alias: Data<string>){
        const keys = Object.keys(alias).map( key => key.replace(/(\S)/g, "\\$1"))

        this.alias = alias
        this.variableFlag = new RegExp(`^\\$([${keys.join('|')}]?)`)
    }

    setOptions(options: EvaluatorOptions){
        this.options = options

        const { source, alias } = options

        this.sourceData = source

        alias && this.setAlias(alias)
    }

    evalItem(source: string): any{
        const { variableFlag, alias } = this
        const isVariable = this.variableFlag.test(source)

        return isVariable ? this.getValueByPath(source.replace(variableFlag, (matched, sub) => sub ? `${alias[sub]}.` : '')) : source
    }

    getValueByPath(path: string): any {
        return getValueByPath(path, this.sourceData)
    }
}