import {toPascalCase} from '../../util'
import logger from '../../util/logger'

type CarReset = boolean | undefined
export type Cache<T> = Map<string, T> | null

export class SimpleManager{
    // 内部缓存对象
    cache: Cache<unknown>
    // 重名时，是否可以被覆盖
    carReset?: CarReset
    // Manager name
    name?: string

    constructor(){
        this.cache = new Map()
    }

    addCaches(cache: object){
        Object.entries(cache).forEach( ([key, item]) => {
            this.add(key, item)
        })
    }

    has(key: string): boolean{
        const {cache} = this

        return !!cache && cache.has(key)
    }

    get(key: string){
        const {cache, name} = this
        const result = cache && cache.get(key)

        if(!result){
            logger.warn(`${this.constructor.name}: get ${name} with key '${key}' error, please make sure you have correctly registered it`)
        }

        return result
    }

    clear(){
        this.cache?.clear()
    }

    add(key: string, item: unknown): boolean{
        const {name, cache } = this
        if(this.has(key) && !this.carReset){
            logger.warn(`${this.constructor.name}: you have already registered the ${name} with key '${key}'`)

            return false
        }

        return !!(cache?.set(key, item))
    }
}