import { Component, SetupContext } from "vue";
import { SimpleManager, Cache } from "./common/simple";
import { NormalizedSchemaNode, ComponentNode } from "../schema";
import { PluginTypes, FORMATTER_SPLITTER } from "../constant";
import { ParserUtil } from "../components/renderer";
import { isString, isFunction } from '../util'

export enum FormatType {
    ToValue = 'toValue',  // 展示值到model上提交给后端的值，一般用于表单具体提交前的格式化
    ToDisplay = 'toDisplay' // Model上的值到显示值的格式化
}

// 组件解析插件
export type FormatterPlugin = {
    name: string,  // 插件名称
    type: PluginTypes.Formatter, // 插件类型
    [FormatType.ToValue]?(input: any, ...args: any[]): any,
    [FormatType.ToDisplay](value: any, ...args: any[]): any,
}

export class FormatterManager extends SimpleManager{
    constructor(){
        super()
        this.name = 'Formatter'
        this.carReset = false
    }
    format(formatInfo: any, input: any, type: FormatType = FormatType.ToDisplay){
        if(!(isString(formatInfo))) return input

        const [formatterType, arg] = formatInfo.split(FORMATTER_SPLITTER)

        if(!formatterType) return input

        const formatter = this.get(formatterType) as FormatterPlugin
        const formatFn = formatter && formatter[type]

        if(!isFunction(formatFn)) return input

        return formatFn(input, arg)
    }
}