// 解析器管理
import { Component, SetupContext } from "vue";
import { SimpleManager, Cache } from "./common/simple";
import { NormalizedSchemaNode, ComponentNode } from "../schema";
import { PluginTypes } from "../constant";
import { ParserUtil } from "../components/renderer";

export type UIName = string

export type UIOptions = {
    name: UIName, // 组件名称
    component: Component, // 组件内容
}

// 组件解析插件
export type ParserPlugin = {
    name: string,  // 插件名称
    type: PluginTypes.Parser, // 插件类型
    ui?: UIName | UIOptions,  // ui组件相关配置
    parse(options: NormalizedSchemaNode): ComponentNode, // 解析逻辑，在setup中被调用，写法类似hooks
    parse(options: NormalizedSchemaNode, context: SetupContext): ComponentNode, // 解析逻辑，在setup中被调用，写法类似hooks
    parse(options: NormalizedSchemaNode, context?: SetupContext, util?: ParserUtil): ComponentNode, // 解析逻辑，在setup中被调用，写法类似hooks
    created?: () => void,
    [key: string]: any,
}

type ParserCache = Cache<ParserPlugin>

export class ParserManager extends SimpleManager{
    constructor(){
        super()
        this.name = 'Parser'
        this.carReset = false
    }
}