/**
 * 通过解析器
 */
import { SetupContext, Ref, ref, computed } from 'vue'
import { ComponentNode, SchemaNodeList, SchemaNode, NormalizedSchemaNode, NormalizedSchema } from '../schema'
import { ParserPlugin } from '../manager/parser'
import { Util } from '../plugins'
import { toPascalCase } from '../util'
import { UITypes, ParserTypes, PluginTypes } from '../constant'

export default {
    name: ParserTypes.Button,  // 插件名称
    type: PluginTypes.Parser,
    ui: UITypes.Button,
    parse: (schema: NormalizedSchemaNode, context: SetupContext, {
        pluginDriver
    }: Util) => {
        const children: Ref<SchemaNodeList<string>> = ref([])
        const onClick = () => pluginDriver.handleAction((schema as SchemaNode).meta)
        const items = computed( () => [{
            key: (schema as NormalizedSchema).uniqueKey,
            ...(schema as SchemaNode).meta,
            onClick,
        }])

        return {
            props: {
                items,
            },
            children
        } as ComponentNode
    },
} as ParserPlugin