/**
 * 通过解析器
 */
import { ref, reactive, SetupContext, computed } from 'vue'
import { ComponentNode, Data, FormItemSchema, SchemaNode, BaseSchema } from '../../schema'
import { ParserPlugin } from '../../manager/parser'
import { ParserTypes, PluginTypes, UITypes } from '../../constant'
import { Util } from '../../plugins'
import { mergeParams } from '../../util/http'
import { FilterMeta } from '../../actions/filter'
import useItemsInit from './useItemsInit'

export type OptionItem = {
    label?: string
    value?: string | number | boolean
    [key: string]: any
}

export type ValidateRule = {
    required?: boolean
    pattern?: string  // 表单校验验证正则字符串
    message: string  // 校验出错的提示信息
}

export type FormPropsMeta = {
    submitTitle?: string
    clearTitle?: string
    actionsShow?: boolean
}

export type FormMethods = {
    submit(): Promise<any>
}

export type FormUIMethods = {
    validate(): Promise<any>
}

export type FormMeta = FilterMeta & FormPropsMeta

const getDefaultValue = (item: FormItemSchema): any => {
    if(item.type === 'checkbox'){
        return []
    }else{
        return ''
    }
}

export default {
    name: ParserTypes.Form,  // 插件名称
    type: PluginTypes.Parser,
    ui: UITypes.Form,
    parse: (schema: SchemaNode, context: SetupContext, {
        pluginDriver
    }: Util) => {
        const isInit = ref(false)
        const {meta} = schema
        const model: Data = reactive(meta?.source || {})
        const { items, initFormItems } = useItemsInit(schema, model, pluginDriver)
        const onUpdateItemValue = (key: string, value: any) => {
            model[key] = value
        }
        const onSubmit = () => {
            mergeParams(meta, model)

            return pluginDriver.handleAction(meta, model)
        }
        const props = computed( () => ({
            ...schema.meta,
            items,
            model,
            onSubmit
        }))

        items.value?.forEach( item => {
            model[item.key] = model[item.key] || item.value || getDefaultValue(item as FormItemSchema)
        })

        initFormItems().then( () => isInit.value = true)

        return {
            isInit,
            props,
            events: {
                onUpdateItemValue
            },
            methods: {
                submit: onSubmit
            }
        } as ComponentNode
    },
} as ParserPlugin