import { reactive, toRef, Ref, computed, ComputedRef } from 'vue'
import { Data, Meta, SchemaNode, FormItems, FormItemSchema } from '../../schema'
import { PluginDriver } from '../../plugins'
import { mergeObjects } from '../../util'
import { ActionTypes, SOURCE_ALIAS } from '../../constant'
import { RequestMeta } from '../../actions'

type ItemMetaRecord = {
    item: FormItemSchema
    request?: RequestMeta
    meta: Meta
}

export default function useItemsInit(schema: SchemaNode, model: Data, pluginDriver: PluginDriver){
    const requstResults: Data = reactive({})
    const itemMetaMap = computed( () => schema.items?.reduce<Data<ItemMetaRecord>>((itemMetaMap, item) => {
        const { request, other: meta } = pluginDriver.destructureMeta(item.meta, ActionTypes.Request)

        request && (itemMetaMap[item.key] = {
            item: item as FormItemSchema,
            request: request as RequestMeta,
            meta
        })

        return itemMetaMap
    }, {}))
    const items: ComputedRef<FormItems> = computed( () => {
        const {items, meta} = schema

        if(!items || !items.length) return []

        const parsedItems: FormItems = pluginDriver.evalOptionsWithVariableKey(items, {
            alias: SOURCE_ALIAS,
            source: {
                source: meta?.source
            }
        })

        return parsedItems.map( item => {
            const {key} = item
            const itemMeta = itemMetaMap.value && itemMetaMap.value[key]

            if(!itemMeta) return item

            const items = requstResults[key] || []
            const { meta } = itemMeta

            mergeObjects(item, {
                items,
            })

            return item
        })
    })
    const initFormItems = async () => {
        const initPromises = itemMetaMap.value && Object.values(itemMetaMap.value).map(async ({request, item}) => {
            const res = await pluginDriver.handleActionByType(ActionTypes.Request, request)

            if(res) requstResults[item.key] = res

            return res
        })

        if(!initPromises) return

        return await Promise.all(initPromises)
    }

    return {
        items,
        initFormItems
    }
}