/**
 * 建站平台框架，对应解析器
 */
import { ref, SetupContext, watch, onUnmounted, Ref, reactive, computed } from 'vue'
import { SchemaNode, ComponentNode, SchemaNodeList, NormalizedSchemaNode, NormalizedSchema, TreeSchema } from '../schema'
import { PluginUtil } from '../plugins'
import { ParserPlugin } from '../manager/parser'
import { ParserTypes, PluginTypes, UITypes } from '../constant'
import { AppConfig } from '../engine'
import useInit from './hooks/useInit'
import { RequestMeta } from '../actions'

type PageResponse = {
    pageJSON?: SchemaNodeList<TreeSchema>
}

export const STACK_DEFAULT_PROPS = {
    direction: 'vertical'
 }

export default {
    name: ParserTypes.Page,  // 插件名称
    type: PluginTypes.Parser,
    ui: UITypes.Stack,
    parse: (schema: NormalizedSchemaNode, context: SetupContext, {
        pluginDriver,
        appConfig,
    }: PluginUtil) => {
        const {kcpApi, pageApiPath, appKey, env} = appConfig as AppConfig
        const children: Ref<SchemaNodeList | undefined> = ref((schema as SchemaNode).items as any)
        const { isInit, initRequest } = useInit(false, pluginDriver)
        const getPageJSON = async () => {
            const initMeta = {
                url: kcpApi + pageApiPath.replace(":appKey", appKey),
                method: 'get',
                params: {
                    pageKey: schema.uniqueKey,
                    env,
                },
                requestPlugin: 'customRequestPlugin'
            } as RequestMeta
            const res = await initRequest(initMeta)

            children.value = (res as PageResponse)?.pageJSON
        }
        const props = computed( () => ({
            ...STACK_DEFAULT_PROPS,
            ...(schema as SchemaNode).meta,
        }))

        watch( () => (schema as NormalizedSchema).uniqueKey, () => {
            isInit.value = false
            children.value = []
            getPageJSON()
        })

        onUnmounted(() => {
            pluginDriver.clearComponents()
        })

        getPageJSON()

        return {
            isInit,
            props,
            children
        } as ComponentNode
    },
} as ParserPlugin