import { reactive, SetupContext } from 'vue'
import { ComponentNode, Data, NormalizedSchemaNode } from '../schema'
import { ParserPlugin } from '../manager/parser'
import { Util } from '../plugins'
import { ParserTypes, PluginTypes, UITypes } from '../constant'
import useInit from './hooks/useInit'
import { mergeObjects, isEmptyObject } from '../util'
import useTitle from './hooks/useTitle'

export type PanelMeta = {
    source?: Data | Array<Data>
}

export default {
    name: ParserTypes.Panel,  // 插件名称
    type: PluginTypes.Parser,
    ui: UITypes.Panel,
    parse: (schema: NormalizedSchemaNode, context: SetupContext, {
        pluginDriver
    }: Util) => {
        const data = reactive(schema.meta?.source || {})
        const {title} = useTitle(schema)
        const {isInit, initRequest} = useInit(isEmptyObject(schema.meta?.source as object), pluginDriver)

        initRequest(schema.meta).then( (res: Data) => {
            mergeObjects(data, res)
        })

        return {
            isInit,
            props: {
                data,
                items: schema.items,
                title,
            }
        } as ComponentNode
    },
} as ParserPlugin