/**
 * 通过解析器
 */
import { toRef, ref, SetupContext, Ref, reactive, computed } from 'vue'
import { SchemaNode, ComponentNode, Meta, Data } from '../../schema'
import { ParserPlugin } from '../../manager/parser'
import { ParserTypes, PluginTypes, UITypes } from '../../constant'
import { Util } from '../../plugins'
import useActions from './useActions'
import { PageChangeTypes } from './usePagination'
import useRequest from './useRequest'

export type TableMethods = {
    refresh(isInit?: boolean): Promise<void>,
    setExtendParams(newV: Data): void
}
export * from './usePagination'

export default {
    name: ParserTypes.Table,  // 插件名称
    type: PluginTypes.Parser,
    ui: UITypes.Table,
    parse: (schema: SchemaNode, context: SetupContext, util: Util) => {
        const meta: Ref<Meta> = toRef(schema, 'meta')
        const { pagination, refresh, data, handlePageChange, setExtendParams, needInit } = useRequest(meta, util)
        const isInit = ref(!needInit.value)
        const { columns } = useActions(schema, util.pluginDriver)
        const onUpdateCurrentPage = (value: number) => handlePageChange(PageChangeTypes.CurrentPage, value)
        const onUpdatePageSize = (value: number) => handlePageChange(PageChangeTypes.PageSize, value)
        const props = computed( () => ({
            columns,
            data,
            pagination,

        }))

        needInit.value && refresh(true).then(() => {
            isInit.value = true
        })

        return {
            isInit,
            props,
            events: {
                'onUpdate:currentPage': onUpdateCurrentPage,
                'onUpdate:pageSize': onUpdatePageSize
            },
            methods: {
                refresh,
                setExtendParams
            } as TableMethods
        } as ComponentNode
    },
} as ParserPlugin