import { computed } from 'vue'
import { PluginDriver, ActionMetaRecords } from '../../plugins'
import { ActionSchema, Data, TreeSchema, TableItemSchema, SchemaNodeList, AllMeta } from '../../schema'
import { MetaRecords } from '../../manager/action'

// 处理表格右侧操作列
export default function useActions(schema: TreeSchema, pluginDriver: PluginDriver){
    const columns = computed( () => schema.items?.map( (item: TableItemSchema): any => {
        const { actions, meta } = item
        const hasActions: boolean = !!(actions && actions?.length)
        const needFormat: boolean = !!(meta && meta.format)

        if(hasActions){
            item.getActions = slotProps => (actions as SchemaNodeList<ActionSchema>).map( (action: ActionSchema): ActionSchema => {
                const evaluatorOptions = {
                    source: slotProps,
                    alias: {'.': 'row'}
                }
                const { meta = {}, ...schema } = pluginDriver.evalOptionsWithVariableKey(action, evaluatorOptions)
                const { other = {}, ...actionMetaRecords }: MetaRecords = pluginDriver.destructureMeta(meta)
                const { confirmable } = meta

                const handleAction = (actionMetaRecords: ActionMetaRecords, row: Data): any => Object.entries(actionMetaRecords).reduce((result, [type, action]) => pluginDriver.handleActionWithVariable({
                    type,
                    action,
                    ...evaluatorOptions,
                }, row), undefined)
                let onClick = (row: Data) => !confirmable && handleAction(actionMetaRecords, row)
                let onConfirm = (row: Data) => confirmable && handleAction(actionMetaRecords, row)

                return {
                    ...other,
                    ...schema,
                    onClick,
                    onConfirm,
                }
            })
        }

        if(needFormat){
            item.formatter = (row: Data, column: Data, cellValue: any) => pluginDriver.formatValue((meta as AllMeta).format, cellValue)
        }

        return item
    }) )

    return {
        columns,
    }
}