import { computed, toRef, reactive, Ref } from 'vue'
import { mergeObjects } from '../../util'
import { Meta, AllMeta } from '../../schema'

export type TableMeta = {
    pageSize?: number // 自定义分页参数的每页条数
    disablePagination?: boolean //是否禁用分页
}

export enum PageChangeTypes{
    CurrentPage = 'currentPage',
    PageSize = 'pageSize',
    Total = 'total'
}

type Pagination = Record<PageChangeTypes, number>

// 分页信息
export function usePagination(meta: Ref<Meta>){
    const paginationParams: Pagination = reactive({
        total: 0,
        currentPage: 1,
        pageSize: (meta.value as TableMeta)?.pageSize || 20
    })
    const disablePagination = toRef(meta.value as AllMeta, 'disablePagination')
    const pagination = computed( () => disablePagination.value ? false : paginationParams )
    const setPagination = (newV: Partial<Pagination>) => {
        mergeObjects(paginationParams, newV)
    }
    const isPaginationChanged = (type: PageChangeTypes, newV: number): boolean => paginationParams[type] !== newV

    return {
        pagination,
        setPagination,
        isPaginationChanged
    }
}