import { computed, ref, reactive, Ref } from 'vue'
import { isFalse, mergeObjects } from '../../util'
import { mergeParams } from '../../util/http'
import { TreeSchema, Meta, AllMeta, Data } from '../../schema'
import { usePagination, PageChangeTypes } from './usePagination'
import { PluginDriver, Util } from '../../plugins'

export default function useRequest(meta: Ref<Meta>, util: Util){
    const { pagination, setPagination, isPaginationChanged } = usePagination(meta)
    const needInit = computed( () => {
        const {source} = meta.value || {}

        return !(source && source.length)
    })
    const data = ref(meta.value?.source || [])
    const extendParams: Data = reactive({})
    const allParams = computed( () => {
        const paginationValue = pagination.value

        if(isFalse(paginationValue)) return extendParams

        const {currentPage, pageSize} = paginationValue

        return mergeObjects(extendParams, {currentPage, pageSize})
    })
    const setExtendParams = (newV: Data) => mergeObjects(extendParams, newV)

    const refresh = async (isInit: boolean = false) => {
        // 初始化时，由ParserComponent管理loading，否则需要自行管理loading
        if(!isInit) util.openLoading()

        const requstMeta = mergeParams(meta.value, allParams.value)
        const responseData = await util.pluginDriver.handleAction(requstMeta)

        if(pagination.value){
            const {
                list,
                currentPage,
                pageSize,
                total
            } = responseData

            data.value = list
            setPagination({
                currentPage,
                pageSize,
                total
            })
        }else{
            data.value = responseData
        }

        if(!isInit) util.closeLoading()
    }
    const handlePageChange = (type: PageChangeTypes, newV: number) => {
        // 分页参数值没有改变时，直接返回
        if(!isPaginationChanged(type, newV)) return

        setPagination({
            [type]: newV,
        })

        refresh()
    }

    return {
        pagination,
        data,
        needInit,
        setExtendParams,
        refresh,
        handlePageChange
    }
}