import { computed, SetupContext, toRefs, Ref, reactive, ref } from 'vue'
import { ComponentNode, NormalizedSchemaNode, SchemaNodeList, Data, ChildSchemaNode, BaseSchema } from '../../schema'
import { ParserPlugin } from '../../manager/parser'
import { Util } from '../../plugins'
import { ParserTypes, PluginTypes, UITypes } from '../../constant'
import { UPDATE_MODEL_HANDLER, MODEL_VALUE } from '../../constant/model'
import { STACK_DEFAULT_PROPS } from '../page'
import useActiveTab from './useActiveTab'

interface TabItemSchema extends BaseSchema{
    default?: boolean // 是否为默认显示的标签页,
    items?: SchemaNodeList<ChildSchemaNode>
}

export type TabItems = Array<TabItemSchema>

type ItemsAndChildrenCollections = {
    items: TabItems,
    children: SchemaNodeList<ChildSchemaNode>
}

export default {
    name: ParserTypes.Tabs,  // 插件名称
    type: PluginTypes.Parser,
    ui: UITypes.Tabs,
    parse: (schema: NormalizedSchemaNode, context: SetupContext, {
        pluginDriver
    }: Util) => {
        const {activeTab, setActiveTab} = useActiveTab(schema.items as TabItems)
        const collections: Ref<ItemsAndChildrenCollections> = computed( () => (schema.items as TabItems|| []).reduce((collections: ItemsAndChildrenCollections, {key, meta, items, ...others}, index) => {
            collections.items.push({
                ...meta,
                ...others,
                key,
            })

            collections.children.push({
                key: `${key}-tab-pane-${index}`,
                type: ParserTypes.Stack,
                meta: {
                    ...STACK_DEFAULT_PROPS,
                },
                items,
                slot: key,
            })

            return collections
        }, {
            items: [],
            children: []
        }))
        const props = computed( () => ({
            [MODEL_VALUE]: activeTab,
            items: collections.value.items
        }) )
        const children = computed( () => collections.value.children )

        return {
            props,
            events: {
                [UPDATE_MODEL_HANDLER]: setActiveTab,
            },
            children,
        } as ComponentNode
    },
} as ParserPlugin