import { Component, Ref } from 'vue'
import { TableMeta, OptionItem, PanelMeta } from '../parsers'
import { RequestMeta, FilterMeta, SubmitMeta, OpenDialogMeta } from '../actions'
import table from '../parsers/table'

type SourceMeta = {
    source?: Data
}

type LabelMeta = {
    label?: string
}

export type ActionMeta = RequestMeta & FilterMeta & SubmitMeta & OpenDialogMeta & SourceMeta & LabelMeta

export type AllMeta = ActionMeta & TableMeta & PanelMeta & Record<string, any>

export type Meta = AllMeta | undefined

export interface BaseSchema {
    uniqueKey?: string
    key: string
    type: string
    label?: string
    meta?: Meta
    show?: boolean
}

export interface TreeSchema<T = SchemaNode> extends BaseSchema{
    items?: SchemaNodeList<T>
}

type FormItemReg = {
    pattern?: string
    message?: string
}

export type  FormItems = Array<OptionItem>

export interface FormItemSchema extends BaseSchema{
    value?: any
    required?: boolean
    reg?: FormItemReg
    items?: FormItems
}

export interface ActionSchema extends BaseSchema{
    onClick?(row: Data, item?: ActionSchema, $event?: any): any
    onConfirm?(row: Data, item?: ActionSchema, $event?: any): any
}

type TableColumnsSlotProps = {
    row: Data,
    [key: string]: any
}

export type Formatter = (...args: any[]) => any

export type GetActions = (slotProps: TableColumnsSlotProps) => SchemaNodeList<ActionSchema>

export interface TableItemSchema extends BaseSchema{
    actions?: SchemaNodeList<ActionSchema>
    getActions?: GetActions
    formatter?: Formatter
}

export type ChildSchemaNode = SchemaNode & {
    slot?: string // 只有ComponentNode的children属性里才有，用于表示当前子节点的slot类型，未定义时代表default
}

export type SchemaNode = TreeSchema & ActionSchema & TableItemSchema & FormItemSchema

export type NormalizedSchema = {
    uniqueKey: string
}

export type NormalizedSchemaNode = NormalizedSchema & SchemaNode

export type SchemaNodeList<T = SchemaNode> = Array<T>

// ActionMeta
type Action = {
    [key: string]: any
}

export type ComponentNode = {
    component?: Component, // 组件,
    isInit?: Ref<boolean>,  // 组件是否初始化完成，需要异步初始化的组件使用
    data?: Record<string, any>, // 组件的data属性，将被添加到属性中
    props?: Ref<Data> | Data, // 组件属性
    events?: Record<string, Action>,  // 组件事件{ change: {url: ''} }
    methods?: Record<string, Function>,
    ref?: string,
    slot?: string,
    children?: Ref<SchemaNodeList<ChildSchemaNode | string>>, // parser只对本组件相关进行解析，children不做转换，递归调用交给ParserComponent去解析
}

export type ComponentNodeList = Array<ComponentNode | string>

type Page = Component

// 组件树通用描述
type ComponentTree = {
    component: Page, //组件
    props: Record<string, any>, // 组件属性
    events: Record<string, Action>,  // 组件事件{ change: {url: ''} }
    ref: string,
    slot: string,
    children: Ref<ComponentNodeList>,
}

export type Data<T = any> = Record<string, T>

export type RestArguments = Array<any>