import Axios, { AxiosInstance, AxiosRequestConfig } from 'axios'
import { Data, Meta } from '../schema'
import { pluginDriver } from '../plugins'
import { mergeObjects, ensureValueByPath } from './index'
import { RequestMeta } from '../actions/request'
import { AppConfig } from '../engine'
import { NEED_LOGIN_CODES } from '../constant/http'
import logger from './logger'

const config = {
    timeout: 5000,
    withCredentials: true,
}

const needLogin = (code: number): boolean =>  NEED_LOGIN_CODES.includes(code)

const isSuccess = (code: number): boolean => code === 0

export type Http = AxiosInstance

export type Response = {
    code: number
    data?: any
    message: string
}

const axios = Axios.create(config)

axios.interceptors.response.use( response => {
    const { data: responseData } = response
    const { code, data = {} } = responseData as Response || {}

    if(needLogin(code)){
        const {location} = window
        const {loginUrl} = pluginDriver.getAppConfig() as AppConfig

        location.replace(`${loginUrl}?returnUrl=${encodeURIComponent(location.href)}`)
    }

    return isSuccess(code) ? data : Promise.reject(responseData)
})

export type HttpRequestConfig = AxiosRequestConfig

export function http(options: HttpRequestConfig): Promise<any>{
    return axios.request(options).then( res => {
        // TODO alias相关逻辑
        return res
    }).catch( err => {
        logger.error('send requst with option', options, 'error: ', err)
        return Promise.reject(err)
    } )
}

const deleteEmptyParams = (params: Data) => Object.entries(params).reduce(( result: Data, [key, value]: [string, any] ) => {
    if(value !== '') result[key] = value

    return result
}, {})

export const mergeParams = (meta: Meta, extendData: Data): Meta  => {
    if(!meta || !meta.url) return

    let { method } = meta as RequestMeta
    const targetKey = !method || method.toUpperCase() === 'POST' ? 'data' : 'params'

    ensureValueByPath(meta, targetKey)

    const target = meta[targetKey] as Data

    mergeObjects(target, extendData)

    meta[targetKey] = deleteEmptyParams(target)

    return meta
}