export * from '../src/manager'
export * from '../src/plugins'
export * from '../src/components/renderer'
export * from '../src/constant'
export * from '../src/schema'
export * from '../src'

declare module qs {
    export type stringify = (params: object, options: object) => any
}

interface RefDataMeta {
    ref: string,
    refData?: {[x:string]: {[x:string]: any}[]}
}
interface DataMeta {
    data: {[x:string]: any}[] | string
    pageSize?: number,
    disablePagination?: boolean
}
export interface ActionMeta {
    label?: string
    modal?: string
    link?: string
    url?: string
    href?: string
    params?: {[x:string]: any}
    method?: string
    confirm?: boolean
    confirmLabel?:string
    form?: string
    rowAction?: string
    componentKey?: string
    show?: string | boolean
}
interface LayoutMeta {
    direction?: string
    width?: string | number
    columnsCount?: number
    rowColCounts?: number []
    labelPosition?: string
    actionsShow?: boolean
    disableGroupCol?: boolean
    accessories?: PageSection []
    style?: {[x:string]: any}
    submitTitle?: string
    clearTitle?: string
    actionDirection?: string
    clearButtonShow?: boolean
    headerBgColor?: string
    headerColor?: string
    block?: boolean
    offset?: number | string
}
interface ItemMeta {
    format?: string
    width?: string
    size?: string;
}

export interface PageSectionItem {
    key: string
    label: string
    id?: boolean
    type?: string
    meta?:  ItemMeta | ActionMeta
    editable?: boolean
    required?: boolean
    reg?: {pattern: string, message: string}
    value?: string
    disabled?: boolean
    showTime?: boolean
    width?: string
    placeholder?: string
    size?: string
}
export interface PageSection {
    type: string
    key: string
    alias?: string
    actionDisabled?: boolean
    items?: (TabItem | PageSectionItem | PageSection)[]
}
export interface TabItem {
    key: string;
    label: string;
    items: (PageSection)[];
    meta?: ActionMeta;
    default?: boolean;
}
export interface MenuItem {
    key: string;
    label: string;
    pageJSON?: PageSection[];
    index?: boolean;
    params?: string[];
    subPages?: (MenuItem)[];
    subs?: MenuItem[];
}
export interface MenuGroup {
    key?: string
    label: string;
    subs: MenuItem[];
}

export interface ConfigType {
    appKey: string;
    appTitle?: string;
    menus?: (MenuGroup | MenuItem) [];
    pageAPI?: string;
    loginUrl?: string;
    logoutBtnCallback?: Function;
}